library("tidyverse")
library("traitdata")
library("Symobio")
library("vegan")

load("data/r_objects/range_maps.RData")

# -------------------------------------------#
#       Assign functional groups          ####
# -------------------------------------------#
# Get taxonomic information for target species
names_unique = unique(range_maps$name_matched[!is.na(range_maps$name_matched)])

species_matched = lapply(names_unique, function(name){
  match_result = Symobio::gbif_match_name(name = name)
  if(match_result$status != "ACCEPTED"){ # Search for accepted name
    match_result = gbif_match_name(usageKey = match_result$acceptedUsageKey)
  }
  match_result$name_orig = name
  return(match_result)
}) %>% 
  bind_rows()

# Assign functional groups
functional_groups = species_matched %>% 
  mutate(
    functional_group = case_when(
      order %in% c("Carnivora", "Artiodactyla", "Cingulata", "Perissodactyla") ~ 1,
      order %in% c("Rodentia", "Didelphimorphia", "Soricomorpha", "Paucituberculata", "Lagomorpha") ~ 2,
      order %in% c("Primates", "Pilosa") ~ 3,
      order %in% c("Chiroptera") ~ 4
    ),
    functional_group = factor(functional_group, labels = c("large ground-dwelling", "small ground-dwelling", "arboreal", "flying"))
  ) %>% 
  dplyr::select(
    name_orig,
    name_matched = species,
    functional_group
  ) %>% 
  distinct()

save(functional_groups, file = "data/r_objects/functional_groups.RData")

# ---------------------------------------------------#
#        Process and merge functional traits      ####
# ---------------------------------------------------#
# Match taxonomic names
traits = traitdata::elton_mammals
names_unique = unique(traits$scientificNameStd)

pb = txtProgressBar(min = 0, max = length(names_unique), style = 3)
trait_names_matched = lapply(seq_along(names_unique), function(i){
  tryCatch({
    name = names_unique[i]
    setTxtProgressBar(pb, i)
    
    match_result = Symobio::gbif_match_name(name = name)
    if(match_result$status != "ACCEPTED"){
      match_result = Symobio::gbif_match_name(usageKey = match_result$acceptedUsageKey)
    }
    match_result$name_orig = name
    return(match_result)  
  }, error = function(e){
    return(NULL)
  })
}) %>% bind_rows()
close(pb)

traits_matched = trait_names_matched %>% 
  dplyr::select("name_orig", "species", "scientificName") %>% 
  dplyr::distinct() %>% 
  dplyr::inner_join(traits, by = c("name_orig" = "scientificNameStd"))

save(traits_matched, file = "data/r_objects/traits_matched.RData")

# Clean trait data
diet_cols = c("Diet.Inv", "Diet.Vend", "Diet.Vect", "Diet.Vfish", "Diet.Vunk", "Diet.Scav", "Diet.Fruit", "Diet.Nect", "Diet.Seed", "Diet.PlantO")
strata_cols = c("ForStrat.Value")
activity_cols = c("Activity.Nocturnal", "Activity.Crepuscular", "Activity.Diurnal")
bodymass_cols = c("BodyMass.Value")

traits_proc = traits_matched %>% 
  mutate(match_genus = stringr::str_detect(species, Genus),
         match_epithet = stringr::str_detect(species, Species)) %>% 
  dplyr::group_by(species) %>% 
  dplyr::group_modify(~ {
    # Only one record with accepted species name
    if (nrow(.x) == 1) { 
      return (.x)
    }
    # If multiple records for accepted species name
    x_mod = .x
    while (nrow(x_mod) > 1){
      # 1. Remove records with mismatched genus name, retry
      if(any(isFALSE(x_mod$match_genus))){
        x_mod = dplyr::filter(x_mod, isTRUE(match_genus))
        next
      }
      
      # 1. Remove records with mismatched epithet name, retry
      if(any(isFALSE(x_mod$match_epithet))){
        x_mod = dplyr::filter(x_mod, isTRUE(match_epithet))
        next
      }
      
      # If there are still multiple records with matching genus and epithet, just take the first record
      x_mod = x_mod[1,]
    }
    
    if(nrow(x_mod) == 1){
      return(x_mod)
    } else {
      return(.x[1,])
    }
  }) %>% 
  dplyr::select(all_of(c("species", diet_cols, strata_cols, activity_cols, bodymass_cols)))

save(traits_proc, file = "data/r_objects/traits_proc.RData")

# ---------------------------------------------------#
#        Calculate functional dissimilarity       ####
# ---------------------------------------------------#
target_species = unique(range_maps$name_matched[!is.na(range_maps$name_matched)])
traits_target = dplyr::filter(traits_proc, species %in% target_species)
  
# some pre-processing
traits_target$`ForStrat.Value` = as.numeric(factor(traits_target$`ForStrat.Value`, levels = c("G", "S", "Ar", "A")))
traits_target$`BodyMass.Value` = scale(log(traits_target$`BodyMass.Value`))

# Calculate distances for trait categories separately
diet_dist = vegan::vegdist(traits_target[,diet_cols], "bray")   # Bray-Curtis
foraging_dist = dist(traits_target[,strata_cols])               # Euclidean
activity_dist = vegan::vegdist(traits_target[,activity_cols], "bray")   # Bray-Curtis
bodymass_dist = dist(traits_target[,bodymass_cols])             # Euclidean

func_dist = (diet_dist + foraging_dist/max(foraging_dist) + activity_dist + bodymass_dist/max(bodymass_dist)) / 4
names(func_dist) = traits_target$species
func_dist = as.matrix(func_dist)

save(func_dist, file = "data/r_objects/func_dist.RData")
