# General packages
library(dplyr)
library(tidyr)
library(ggplot2)

# Geo packages
library(terra)
library(sf)
library(geos)
library(blockCV)

source("R/utils.R")

load("data/r_objects/range_maps.RData")
load("data/r_objects/occs_final.RData")
load("data/r_objects/sa_polygon.RData")

# ---------------------------------------------------------------------------#
# Prepare Geodata                                                         ####
# ---------------------------------------------------------------------------#
proj_string = "+proj=tcea +lon_0=-58.0704167 +datum=WGS84 +units=m +no_defs" 

target_species = unique(occs_final$species)

raster_filepaths = list.files("~/symobio-modeling/data/geospatial/raster/", full.names = T) %>% 
  stringr::str_sort(numeric = T) 

sa_polygon = st_transform(sa_polygon, proj_string)
occs_final = st_transform(occs_final, proj_string)
range_maps = st_transform(range_maps, proj_string)
raster_data = terra::rast(raster_filepaths) %>% 
  terra::project(proj_string) 

# ---------------------------------------------------------------------------#
# Sample random absences                                                  ####
#                                                                            #
# Absences are sampled at random                                             #
# (1) within the defined sampling area (same number as presences)            #
# (2) outside the defined sampling area (100 records)                        #
# ---------------------------------------------------------------------------#
species_processed = list.files("data/r_objects/pa_sampling/", pattern = "RData") %>% 
  stringr::str_remove(".RData")

for(species in setdiff(target_species, species_processed)){
  print(species)
  # Prepare species occurrence data #####
  occs_spec = dplyr::filter(occs_final, species == !!species)
  if(nrow(occs_spec) <= 1){
    warning("Skipping species ", species, ": Can't process species with a single presence record." )
  }
  
  occs_multipoint = occs_spec %>% 
    summarise(geometry = st_combine(geometry))
  
  range_spec = dplyr::filter(range_maps, name_matched == !!species) %>% 
    rename(species = name_matched)
  
  # Define core sampling region
  # 1. Union all points from range polygon and occurrences records
  # 2. Build concave polygon from unioned points
  # 3. Buffer by 50 km to extend sampling region into new environments
  # 4. Crop by South America outline
  # 5. Crop by buffered presence points
  sampling_region = st_cast(range_spec, "MULTIPOINT") %>% 
    st_union(occs_multipoint) %>% 
    geos::geos_concave_hull(ratio = 0.3) %>% # Concave hull in sf requires geos 3.11
    st_as_sf() %>% 
    st_set_crs(proj_string) %>% 
    st_buffer(dist = 50000) %>% 
    st_intersection(sa_polygon) %>% 
    st_difference(st_buffer(occs_multipoint, dist = 50000))
  
  # Sample balanced number of pseudo absences inside core region
  abs_spec_list = list()
  
  samples_required = nrow(occs_spec)
  while(samples_required != 0){
    sample_points = sf::st_sample(sampling_region, size = samples_required)
    env_sample = terra::extract(raster_data, terra::vect(sample_points), ID = FALSE)
    
    sample_abs = st_sf(env_sample, geometry = sample_points) %>% 
      drop_na() %>% 
      dplyr::mutate(
        record_type = "core"
      )
    
    abs_spec_list[[length(abs_spec_list)+1]] = sample_abs
    
    samples_required = samples_required - nrow(sample_abs) # Sometimes there are no env data for sample points, so keep sampling
  }
  
  # Sample 100 pseudo absences outside core region
  background_region = st_difference(sa_polygon, sampling_region)
  samples_required = 100
  while(samples_required != 0){
    sample_points = sf::st_sample(background_region, size = samples_required)
    env_sample = terra::extract(raster_data, terra::vect(sample_points), ID = FALSE)
    
    sample_abs = st_sf(env_sample, geometry = sample_points) %>% 
      drop_na() %>% 
      dplyr::mutate(
        record_type = "background"
      )
    
    abs_spec_list[[length(abs_spec_list)+1]] = sample_abs
    
    samples_required = samples_required - nrow(sample_abs) # Sometimes there are no env data for sample points, so keep sampling
  }
  
  # Consolidate absences
  abs_spec = bind_rows(abs_spec_list)
  abs_spec = abs_spec %>% 
    bind_cols(
      st_coordinates(st_transform(abs_spec, "EPSG:4326")) 
    ) %>% 
    dplyr::rename(
      longitude = X,
      latitude = Y
    ) %>% 
    dplyr::mutate(
      species = !!species,
      present = 0
    ) 
  
  # Create presence-absence dataframe
  pa_spec = occs_spec %>% 
    dplyr::mutate(
      present = 1,
      record_type = "core"
    ) %>% 
    bind_rows(abs_spec)
  

  # Define cross-validation folds
  pa_spec_core = dplyr::filter(pa_spec, record_type == "core")
  pa_spec_background = dplyr::filter(pa_spec, record_type == "background")
  
  folds = tryCatch({
    spatial_folds = suppressMessages(
      blockCV::cv_spatial(
        pa_spec_core,
        column = "present",
        k = 5,
        progress = F, plot = F, report = F
      )
    )
    
    spatial_folds$folds_ids
  }, error = function(e){
    NA
  })
  
  pa_spec = pa_spec_core %>% 
    dplyr::mutate(fold_individual = folds) %>% 
    bind_rows(pa_spec_background) %>% 
    group_by(present) %>%
    mutate(weight = 1/n()) %>%
    ungroup()
  
  # Plot result
  ref = st_bbox(sampling_region)[c(1,3,2,4)]
  ggplot() +
    ggtitle(species) +
    geom_sf(data = sa_polygon) +
    geom_sf(data = range_spec, aes(fill = "IUCN range"), alpha = 0.1, color = NA, show.legend = "fill") +
    geom_sf(data = sampling_region, aes(fill = "Sampling region"), alpha = 0.1, color = NA, show.legend = "fill") +
    geom_sf(data = pa_spec, aes(color = as.factor(present), shape = as.factor(record_type)), size = 0.7) +
    scale_fill_manual(name = "Polygons", values = c("IUCN range" = "red", "Sampling region" = "blue")) +
    scale_color_discrete(name = "Presence") +
    scale_shape_discrete(name = "Record type") +
    theme_minimal()
  
  try({
    ggsave(paste0(species, ".pdf"), path = "plots/pa_sampling", device = "pdf", scale = 3)
  })
  
  save(pa_spec, file = paste0("data/r_objects/pa_sampling/", species, ".RData"))
}
