library(tidyverse)
library(sf)
library(terra)

source("R/utils.R")

load("data/r_objects/model_data.RData")
load("data/r_objects/sa_polygon.RData")

sa_polygon = sf::st_transform(sa_polygon, crs(model_data))

# ---------------------------------------#
# Plot all presences in the dataset   ####
# ---------------------------------------#
# Prepare data
data_extent = ext(model_data)

presences = model_data %>% 
  dplyr::filter(present == 1)

template_raster <- terra::rast(data_extent, resolution = 1000)  # 1000m = 1km
crs(template_raster) <- st_crs(model_data)$wkt

point_counts <- terra::rasterize(
  vect(presences),          
  template_raster,           
  field = NULL,              
  fun = "count",             
  background = 0             
) %>% 
  terra::mask(vect(sa_polygon))

length(which(values(point_counts) > 0)) 
# --> only 36858 raster cells contain at least 1 presence record 

# Create plot
ggplot() +
  tidyterra::geom_spatraster(data = point_counts, maxcell = 5e7) +
  scale_fill_gradientn(
    colors = c("black", "yellow", "darkred"),
    values = scales::rescale(c(0, 1, max(point_counts_df$count, na.rm = TRUE))),
    breaks = c(0, 1, max(point_counts_df$count, na.rm = TRUE)),
    na.value = "transparent"
  ) +
  geom_sf(data = sa_polygon, fill = NA, color = "gray", size = 0.5) +
  theme_minimal() +
  coord_sf() +
  labs(title = "Coordinate distribution and density")

ggsave("coordinate_density.pdf", path = "plots/", device = "pdf", scale = 4)

# ---------------------------------------#
# Explore folds assignment            ####
# ---------------------------------------#
# Prepare data
cv_plot(spatial_folds)
ggsave(filename = "plots/publication/global_folds_map.pdf", device = "pdf", scale = 2.5)

folds_summary = model_data %>% 
  dplyr::filter(record_type == "core") %>% 
  sf::st_drop_geometry() %>% 
  dplyr::group_by(species, fold_global) %>% 
  dplyr::summarise(
    n_occ = length(which(present == 1)),
    n_abs = length(which(present == 0))
  ) 

folds_uniform = folds_summary %>% 
  dplyr::group_by(species, fold_global) %>% 
  dplyr::summarise(
    is_uniform = n_occ == 0 | n_abs == 0
  ) %>% 
  dplyr::group_by(species) %>% 
  dplyr::summarise(
    folds_total = n(),
    folds_uniform = length(which(is_uniform)),
    folds_mixed = folds_total - folds_uniform
  ) %>% 
  dplyr::ungroup()

# Create plot
plot_1 = ggplot(folds_uniform, aes(x=folds_total)) + 
  geom_histogram(binwidth=0.5, fill = "orange") + 
  labs(title="Number of folds per species", x="Count", y = "Number of species") +
  theme_minimal()
plot_2 = ggplot(folds_uniform, aes(x=folds_uniform)) + 
  geom_histogram(binwidth=0.5, fill = "violet") + 
  labs(title="Number of uniform folds per species", x="Count", y = "Number of species",
       subtitle="Uniform folds contain only absences or only presences and cannot be used in model evaluation.") +
  theme_minimal()
plot_3 = ggplot(folds_uniform, aes(x=folds_mixed)) + 
  geom_histogram(binwidth=0.5, fill = "blue") + 
  labs(title="Number of mixed folds per species", x="Count", y = "Number of species",
       subtitle="Mixed folds both absences and presences and can be used in model evaluation.") +
  theme_minimal()

plot_combined = grid.arrange(plot_1, plot_2, plot_3, ncol = 1)
ggsave(filename = "plots/publication/global_folds_histogram.pdf", plot_combined, device = "pdf", height = 10, width = 7)


